@@ -15,17 +15,15 @@ import click_completion
 import crayons
 import dotenv
 import delegator
-import pexpect
+from .vendor import pexpect
 import pipfile
-import pipdeptree
-from pipreqs import pipreqs
 from blindspin import spinner
-
 from requests.packages import urllib3
 from requests.packages.urllib3.exceptions import InsecureRequestWarning
+import six
 
 from .cmdparse import ScriptEmptyError
-from .project import Project
+from .project import Project, SourceNotFound
 from .utils import (
     convert_deps_from_pip,
     convert_deps_to_pip,
@@ -48,13 +46,13 @@ from .utils import (
     is_star,
     TemporaryDirectory,
     rmtree,
+    split_argument,
 )
 from .import pep508checker, progress
 from .environments import (
     PIPENV_COLORBLIND,
     PIPENV_NOSPIN,
     PIPENV_SHELL_FANCY,
-    PIPENV_VENV_IN_PROJECT,
     PIPENV_TIMEOUT,
     PIPENV_SKIP_VALIDATION,
     PIPENV_HIDE_EMOJIS,
@@ -71,6 +69,7 @@ from .environments import (
     PIPENV_DOTENV_LOCATION,
     PIPENV_SHELL,
     PIPENV_PYTHON,
+    PIPENV_VIRTUALENV,
 )
 
 # Backport required for earlier versions of Python.
@@ -116,8 +115,8 @@ if PIPENV_NOSPIN:
 
 
 def which(command, location=None, allow_global=False):
-    if location is None:
-        location = project.virtualenv_location
+    if not allow_global and location is None:
+        location = project.virtualenv_location or os.environ.get('VIRTUAL_ENV')
     if not allow_global:
         if os.name == 'nt':
             p = find_windows_executable(
@@ -128,6 +127,11 @@ def which(command, location=None, allow_global=False):
     else:
         if command == 'python':
             p = sys.executable
+    if not os.path.exists(p):
+        if command == 'python':
+            p = sys.executable or system_which('python')
+        else:
+            p = system_which(command)
     return p
 
 
@@ -183,7 +187,7 @@ def cleanup_virtualenv(bare=True):
 
 def import_requirements(r=None, dev=False):
     import pip9
-    from pip9.req.req_file import parse_requirements
+    from .vendor.pip9.req.req_file import parse_requirements
 
     # Parse requirements.txt file with Pip's parser.
     # Pip requires a `PipSession` which is a subclass of requests.Session.
@@ -234,6 +238,7 @@ def ensure_environment():
 
 
 def import_from_code(path='.'):
+    from pipreqs import pipreqs
     rs = []
     try:
         for r in pipreqs.get_all_imports(path):
@@ -246,11 +251,22 @@ def import_from_code(path='.'):
         return []
 
 
-def ensure_pipfile(validate=True, skip_requirements=False):
+def ensure_pipfile(validate=True, skip_requirements=False, system=False):
     """Creates a Pipfile for the project, if it doesn't exist."""
-    global USING_DEFAULT_PYTHON
+    global USING_DEFAULT_PYTHON, PIPENV_VIRTUALENV
     # Assert Pipfile exists.
+    python = which('python') if not (USING_DEFAULT_PYTHON or system) else None
     if project.pipfile_is_empty:
+        # Show an error message and exit if system is passed and no pipfile exists
+        if system and not PIPENV_VIRTUALENV:
+            click.echo(
+                '{0}: --system is intended to be used for pre-existing Pipfile '
+                'installation, not installation of specific packages. Aborting.'.format(
+                    crayons.red('Warning', bold=True)
+                ),
+                err=True,
+            )
+            sys.exit(1)
         # If there's a requirements file, but no Pipfile...
         if project.requirements_exists and not skip_requirements:
             click.echo(
@@ -260,7 +276,6 @@ def ensure_pipfile(validate=True, skip_requirements=False):
                 )
             )
             # Create a Pipfile...
-            python = which('python') if not USING_DEFAULT_PYTHON else None
             project.create_pipfile(python=python)
             with spinner():
                 # Import requirements.txt.
@@ -284,7 +299,6 @@ def ensure_pipfile(validate=True, skip_requirements=False):
                 err=True,
             )
             # Create the pipfile if it doesn't exist.
-            python = which('python') if not USING_DEFAULT_PYTHON else False
             project.create_pipfile(python=python)
     # Validate the Pipfile's contents.
     if validate and project.virtualenv_exists and not PIPENV_SKIP_VALIDATION:
@@ -631,7 +645,7 @@ def ensure_project(
                         click.echo(crayons.red('Deploy aborted.'), err=True)
                         sys.exit(1)
     # Ensure the Pipfile exists.
-    ensure_pipfile(validate=validate, skip_requirements=skip_requirements)
+    ensure_pipfile(validate=validate, skip_requirements=skip_requirements, system=system)
 
 
 def shorten_path(location, bold=False):
@@ -767,7 +781,10 @@ def do_install_dependencies(
                 l[i] = list(l[i])
                 if '--hash' in l[i][0]:
                     l[i][0] = (l[i][0].split('--hash')[0].strip())
+        index_args = prepare_pip_source_args(project.sources)
+        index_args = ' '.join(index_args).replace(' -', '\n-')
         # Output only default dependencies
+        click.echo(index_args)
         if not dev:
             click.echo('\n'.join(d[0] for d in sorted(deps_list)))
             sys.exit(0)
@@ -782,12 +799,8 @@ def do_install_dependencies(
     for dep, ignore_hash, block in deps_list_bar:
         if len(procs) < PIPENV_MAX_SUBPROCESS:
             # Use a specific index, if specified.
-            index = None
-            if ' -i ' in dep:
-                dep, index = dep.split(' -i ')
-                dep = '{0} {1}'.format(dep, ' '.join(index.split()[1:])).strip(
-                )
-                index = index.split()[0]
+            dep, index = split_argument(dep, short='i', long_='index')
+            dep, extra_index = split_argument(dep, long_='extra-index-url')
             # Install the module.
             c = pip_install(
                 dep,
@@ -798,6 +811,7 @@ def do_install_dependencies(
                 block=block,
                 index=index,
                 requirements_dir=requirements_dir,
+                extra_indexes=extra_index,
             )
             c.dep = dep
             c.ignore_hash = ignore_hash
@@ -816,12 +830,9 @@ def do_install_dependencies(
         for dep, ignore_hash in progress.bar(
             failed_deps_list, label=INSTALL_LABEL2
         ):
-            index = None
-            if ' -i ' in dep:
-                dep, index = dep.split(' -i ')
-                dep = '{0} {1}'.format(dep, ' '.join(index.split()[1:])).strip(
-                )
-                index = index.split()[0]
+            # Use a specific index, if specified.
+            dep, index = split_argument(dep, short='i', long_='index')
+            dep, extra_index = split_argument(dep, long_='extra-index-url')
             # Install the module.
             c = pip_install(
                 dep,
@@ -831,6 +842,7 @@ def do_install_dependencies(
                 verbose=verbose,
                 index=index,
                 requirements_dir=requirements_dir,
+                extra_indexes=extra_index,
             )
             # The Installation failed...
             if c.return_code != 0:
@@ -871,7 +883,7 @@ def do_create_virtualenv(python=None, site_packages=False):
         err=True,
     )
     # The user wants the virtualenv in the project.
-    if PIPENV_VENV_IN_PROJECT:
+    if project.is_venv_in_project():
         cmd = [
             'virtualenv',
             project.virtualenv_location,
@@ -921,7 +933,7 @@ def do_create_virtualenv(python=None, site_packages=False):
             sys.exit(1)
     click.echo(crayons.blue(c.out), err=True)
     # Enable site-packages, if desired...
-    if not PIPENV_VENV_IN_PROJECT and site_packages:
+    if not project.is_venv_in_project() and site_packages:
         click.echo(
             crayons.normal(u'Making site-packages available…', bold=True),
             err=True,
@@ -1051,7 +1063,7 @@ def do_lock(
     # TODO: be smarter about this.
     vcs_deps = convert_deps_to_pip(project.vcs_dev_packages, project, r=False)
     pip_freeze = delegator.run(
-        '{0} freeze'.format(escape_grouped_arguments(which_pip()))
+        '{0} freeze'.format(escape_grouped_arguments(which_pip(allow_global=system)))
     ).out
     if vcs_deps:
         for line in pip_freeze.strip().split('\n'):
@@ -1116,9 +1128,14 @@ def do_lock(
     # Add refs for VCS installs.
     # TODO: be smarter about this.
     vcs_deps = convert_deps_to_pip(project.vcs_packages, project, r=False)
-    pip_freeze = delegator.run('{0} freeze'.format(which_pip())).out
+    pip_freeze = delegator.run('{0} freeze'.format(escape_grouped_arguments(which_pip(allow_global=system)))).out
     for dep in vcs_deps:
         for line in pip_freeze.strip().split('\n'):
+            # if the line doesn't match a vcs dependency in the Pipfile,
+            # ignore it
+            if not any(dep in line for dep in vcs_deps):
+                continue
+
             try:
                 installed = convert_deps_from_pip(line)
                 name = list(installed.keys())[0]
@@ -1136,11 +1153,12 @@ def do_lock(
             for package_specified in section:
                 norm_name = pep423_name(package_specified)
                 if not is_pinned(section[package_specified]):
-                    lockfile[section_name][norm_name] = cached_lockfile[
-                        section_name
-                    ][
-                        norm_name
-                    ]
+                    if norm_name in cached_lockfile[section_name]:
+                        lockfile[section_name][norm_name] = cached_lockfile[
+                            section_name
+                        ][
+                            norm_name
+                        ]
     # Overwrite any develop packages with default packages.
     for default_package in lockfile['default']:
         if default_package in lockfile['develop']:
@@ -1285,7 +1303,7 @@ def do_init(
                 cleanup_virtualenv(bare=False)
                 sys.exit(1)
     # Ensure the Pipfile exists.
-    ensure_pipfile()
+    ensure_pipfile(system=system)
     if not requirements_dir:
         requirements_dir = TemporaryDirectory(
             suffix='-requirements', prefix='pipenv-'
@@ -1308,6 +1326,20 @@ def do_init(
                 )
                 requirements_dir.cleanup()
                 sys.exit(1)
+            elif system or allow_global:
+                click.echo(
+                    crayons.red(
+                        u'Pipfile.lock ({0}) out of date, but installation '
+                        u'uses {1}... re-building lockfile must happen in '
+                        u'isolation. Please rebuild lockfile in a virtualenv. '
+                        u'Continuing anyway...'.format(
+                            crayons.white(old_hash[-6:]),
+                            crayons.white('--system')
+                        ),
+                        bold=True,
+                    ),
+                    err=True,
+                )
             else:
                 click.echo(
                     crayons.red(
@@ -1321,16 +1353,28 @@ def do_init(
                 do_lock(system=system, pre=pre, keep_outdated=keep_outdated)
     # Write out the lockfile if it doesn't exist.
     if not project.lockfile_exists and not skip_lock:
-        click.echo(
-            crayons.normal(u'Pipfile.lock not found, creating…', bold=True),
-            err=True,
-        )
-        do_lock(
-            system=system,
-            pre=pre,
-            keep_outdated=keep_outdated,
-            verbose=verbose,
-        )
+        if system or allow_global:
+            click.echo(
+                '{0}: --system is intended to be used for Pipfile installation, '
+                'not installation of specific packages. Aborting.'.format(
+                    crayons.red('Warning', bold=True)
+                ),
+                err=True,
+            )
+            click.echo('See also: --deploy flag.', err=True)
+            requirements_dir.cleanup()
+            sys.exit(1)
+        else:
+            click.echo(
+                crayons.normal(u'Pipfile.lock not found, creating…', bold=True),
+                err=True,
+            )
+            do_lock(
+                system=system,
+                pre=pre,
+                keep_outdated=keep_outdated,
+                verbose=verbose,
+            )
     do_install_dependencies(
         dev=dev,
         requirements=requirements,
@@ -1358,6 +1402,7 @@ def pip_install(
     pre=False,
     selective_upgrade=False,
     requirements_dir=None,
+    extra_indexes=None,
 ):
     import pip9
 
@@ -1406,58 +1451,70 @@ def pip_install(
             src = ''
     else:
         src = ''
+
     # Try installing for each source in project.sources.
     if index:
+        if not is_valid_url(index):
+            index = project.find_source(index).get('url')
         sources = [{'url': index}]
+        if extra_indexes:
+            if isinstance(extra_indexes, six.string_types):
+                extra_indexes = [extra_indexes,]
+            for idx in extra_indexes:
+                try:
+                    extra_src = project.find_source(idx).get('url')
+                except SourceNotFound:
+                    extra_src = idx
+                if extra_src != index:
+                    sources.append({'url': extra_src})
+        else:
+            for idx in project.pipfile_sources:
+                if idx['url'] != sources[0]['url']:
+                    sources.append({'url': idx['url']})
     else:
-        sources = project.sources
-    for source in sources:
-        if package_name.startswith('-e '):
-            install_reqs = ' -e "{0}"'.format(package_name.split('-e ')[1])
-        elif r:
-            install_reqs = ' -r {0}'.format(r)
-        else:
-            install_reqs = ' "{0}"'.format(package_name)
-        # Skip hash-checking mode, when appropriate.
-        if r:
-            with open(r) as f:
-                if '--hash' not in f.read():
-                    ignore_hashes = True
-        else:
-            if '--hash' not in install_reqs:
+        sources = project.pipfile_sources
+    if package_name.startswith('-e '):
+        install_reqs = ' -e "{0}"'.format(package_name.split('-e ')[1])
+    elif r:
+        install_reqs = ' -r {0}'.format(escape_grouped_arguments(r))
+    else:
+        install_reqs = ' "{0}"'.format(package_name)
+    # Skip hash-checking mode, when appropriate.
+    if r:
+        with open(r) as f:
+            if '--hash' not in f.read():
                 ignore_hashes = True
-        verbose_flag = '--verbose' if verbose else ''
-        if not ignore_hashes:
-            install_reqs += ' --require-hashes'
-        no_deps = '--no-deps' if no_deps else ''
-        pre = '--pre' if pre else ''
-        quoted_pip = which_pip(allow_global=allow_global)
-        quoted_pip = escape_grouped_arguments(quoted_pip)
-        upgrade_strategy = '--upgrade --upgrade-strategy=only-if-needed' if selective_upgrade else ''
-        pip_command = '{0} install {4} {5} {6} {7} {3} {1} {2} --exists-action w'.format(
-            quoted_pip,
-            install_reqs,
-            ' '.join(prepare_pip_source_args([source])),
-            no_deps,
-            pre,
-            src,
-            verbose_flag,
-            upgrade_strategy,
-        )
-        if verbose:
-            click.echo('$ {0}'.format(pip_command), err=True)
-        c = delegator.run(pip_command, block=block)
-        if c.return_code == 0:
-            break
-
-    # Return the result of the first one that runs ok, or the last one that didn't work.
+    else:
+        if '--hash' not in install_reqs:
+            ignore_hashes = True
+    verbose_flag = '--verbose' if verbose else ''
+    if not ignore_hashes:
+        install_reqs += ' --require-hashes'
+    no_deps = '--no-deps' if no_deps else ''
+    pre = '--pre' if pre else ''
+    quoted_pip = which_pip(allow_global=allow_global)
+    quoted_pip = escape_grouped_arguments(quoted_pip)
+    upgrade_strategy = '--upgrade --upgrade-strategy=only-if-needed' if selective_upgrade else ''
+    pip_command = '{0} install {4} {5} {6} {7} {3} {1} {2} --exists-action w'.format(
+        quoted_pip,
+        install_reqs,
+        ' '.join(prepare_pip_source_args(sources)),
+        no_deps,
+        pre,
+        src,
+        verbose_flag,
+        upgrade_strategy,
+    )
+    if verbose:
+        click.echo('$ {0}'.format(pip_command), err=True)
+    c = delegator.run(pip_command, block=block)
     return c
 
 
 def pip_download(package_name):
     for source in project.sources:
         cmd = '{0} download "{1}" -i {2} -d {3}'.format(
-            delegator.run(which_pip()),
+            escape_grouped_arguments(which_pip()),
             package_name,
             source['url'],
             project.download_location,
@@ -1736,7 +1793,7 @@ def do_install(
         keep_outdated = project.settings.get('keep_outdated')
     remote = requirements and is_valid_url(requirements)
     # Warn and exit if --system is used without a pipfile.
-    if system and package_name:
+    if system and package_name and not PIPENV_VIRTUALENV:
         click.echo(
             '{0}: --system is intended to be used for Pipfile installation, '
             'not installation of specific packages. Aborting.'.format(
@@ -1831,6 +1888,19 @@ def do_install(
     more_packages = list(more_packages)
     if package_name == '-e':
         package_name = ' '.join([package_name, more_packages.pop(0)])
+    # capture indexes and extra indexes
+    line = [package_name] + more_packages
+    index_indicators = ['-i', '--index', '--extra-index-url']
+    index, extra_indexes = None, None
+    if more_packages and any(more_packages[0].startswith(s) for s in index_indicators):
+        line, index = split_argument(' '.join(line), short='i', long_='index')
+        line, extra_indexes = split_argument(line, long_='extra-index-url')
+        package_names = line.split()
+        package_name = package_names[0]
+        if len(package_names) > 1:
+            more_packages = package_names[1:]
+        else:
+            more_packages = []
     # Capture . argument and assign it to nothing
     if package_name == '.':
         package_name = False
@@ -1910,6 +1980,8 @@ def do_install(
                 verbose=verbose,
                 pre=pre,
                 requirements_dir=requirements_directory.name,
+                index=index,
+                extra_indexes=extra_indexes,
             )
             # Warn if --editable wasn't passed.
             try:
@@ -1979,6 +2051,7 @@ def do_install(
             verbose=verbose,
             keep_outdated=keep_outdated,
             requirements_dir=requirements_directory,
+            deploy=deploy,
         )
         requirements_directory.cleanup()
 
@@ -2036,8 +2109,9 @@ def do_uninstall(
         sys.exit(1)
     for package_name in package_names:
         click.echo(u'Un-installing {0}…'.format(crayons.green(package_name)))
-        cmd = '"{0}" uninstall {1} -y'.format(
-            which_pip(allow_global=system), package_name
+        cmd = '{0} uninstall {1} -y'.format(
+            escape_grouped_arguments(which_pip(allow_global=system)),
+            package_name,
         )
         if verbose:
             click.echo('$ {0}'.format(cmd))
@@ -2069,7 +2143,7 @@ def do_uninstall(
 
 
 def do_shell(three=None, python=False, fancy=False, shell_args=None):
-    from pipenv.patched.pew import pew
+    from .patched.pew import pew
 
     # Ensure that virtualenv is available.
     ensure_project(three=three, python=python, validate=False)
@@ -2107,7 +2181,7 @@ def do_shell(three=None, python=False, fancy=False, shell_args=None):
         args = []
     # Standard (properly configured shell) mode:
     else:
-        if PIPENV_VENV_IN_PROJECT:
+        if project.is_venv_in_project():
             # use .venv as the target virtualenv name
             workon_name = '.venv'
         else:
@@ -2119,7 +2193,7 @@ def do_shell(three=None, python=False, fancy=False, shell_args=None):
     terminal_dimensions = get_terminal_size()
     try:
         with temp_environ():
-            if PIPENV_VENV_IN_PROJECT:
+            if project.is_venv_in_project():
                 os.environ['WORKON_HOME'] = project.project_directory
             c = pexpect.spawn(
                 cmd,
@@ -2133,7 +2207,7 @@ def do_shell(three=None, python=False, fancy=False, shell_args=None):
         # import subprocess
         # Tell pew to use the project directory as its workon_home
         with temp_environ():
-            if PIPENV_VENV_IN_PROJECT:
+            if project.is_venv_in_project():
                 os.environ['WORKON_HOME'] = project.project_directory
             pew.workon_cmd([workon_name])
             sys.exit(0)
@@ -2329,6 +2403,7 @@ def do_check(three=None, python=False, system=False, unused=False, args=None):
 
 
 def do_graph(bare=False, json=False, reverse=False):
+    import pipdeptree
     try:
         python_path = which('python')
     except AttributeError:
@@ -2442,7 +2517,7 @@ def do_clean(
     ensure_lockfile()
     installed_packages = filter(
         None,
-        delegator.run('{0} freeze'.format(which('pip'))).out.strip().split(
+        delegator.run('{0} freeze'.format(which_pip())).out.strip().split(
             '\n'
         ),
     )
@@ -2485,7 +2560,7 @@ def do_clean(
             # Uninstall the package.
             c = delegator.run(
                 '{0} uninstall {1} -y'.format(
-                    which('pip'), apparent_bad_package
+                    which_pip(), apparent_bad_package
                 )
             )
             if c.return_code != 0:
